@@ -43,7 +43,7 @@
 _CCCL_BEGIN_NAMESPACE_CUDA
 
 //! @rst
-//! .. _cudax-containers-uninitialized-async-buffer:
+//! .. _libcudacxx-containers-uninitialized-async-buffer:
 //!
 //! Uninitialized stream-ordered type-safe memory storage
 //! ------------------------------------------------------
@@ -205,7 +205,7 @@ public:
       : __mr_(::cuda::std::move(__mr))
       , __stream_(__stream)
       , __count_(__count)
-      , __buf_(__count_ == 0 ? nullptr : __mr_.allocate(__stream_, __get_allocation_size(__count_)))
+      , __buf_(__count_ == 0 ? nullptr : __mr_.allocate(__stream_, __get_allocation_size(__count_), alignof(_Tp)))
   {}
 
   _CCCL_HIDE_FROM_ABI __uninitialized_async_buffer(const __uninitialized_async_buffer&)            = delete;
@@ -247,7 +247,7 @@ public:
 
     if (__buf_)
     {
-      __mr_.deallocate(__stream_, __buf_, __get_allocation_size(__count_));
+      __mr_.deallocate(__stream_, __buf_, __get_allocation_size(__count_), alignof(_Tp));
     }
     __mr_     = ::cuda::std::move(__other.__mr_);
     __stream_ = ::cuda::std::exchange(__other.__stream_, ::cuda::stream_ref{::cudaStream_t{}});
@@ -267,7 +267,7 @@ public:
   {
     if (__buf_)
     {
-      __mr_.deallocate(__stream, __buf_, __get_allocation_size(__count_));
+      __mr_.deallocate(__stream, __buf_, __get_allocation_size(__count_), alignof(_Tp));
       __buf_   = nullptr;
       __count_ = 0;
     }
