@@ -80,13 +80,13 @@
 
 // We can only expose constexpr allocations if the compiler supports it
 // For now disable constexpr allocation support until we can actually use
-#if 0 && defined(__cpp_constexpr_dynamic_alloc) && defined(__cpp_lib_constexpr_dynamic_alloc) && _CCCL_STD_VER >= 2020 \
-  && !_CCCL_COMPILER(NVRTC)
+#if 0 && __cpp_constexpr_dynamic_alloc >= 201907L && __cpp_lib_constexpr_dynamic_alloc >= 201907L \
+  && _CCCL_STD_VER >= 2020 && !_CCCL_COMPILER(NVRTC)
 #  define _CCCL_HAS_CONSTEXPR_ALLOCATION
 #  define _CCCL_CONSTEXPR_CXX20_ALLOCATION constexpr
-#else // ^^^ __cpp_constexpr_dynamic_alloc ^^^ / vvv !__cpp_constexpr_dynamic_alloc vvv
+#else // ^^^ has constexpr allocations ^^^ / vvv no constexpr allocations vvv
 #  define _CCCL_CONSTEXPR_CXX20_ALLOCATION
-#endif
+#endif // ^^^ no constexpr allocations ^^^
 
 // Enable removed C++17 features
 #if defined(_LIBCUDACXX_ENABLE_CXX17_REMOVED_FEATURES)
@@ -103,7 +103,7 @@
 
 // Third party libraries
 
-#if _CCCL_HAS_INCLUDE(<dlpack/dlpack.h>) && !_CCCL_COMPILER(NVRTC) && !defined(CCCL_DISABLE_DLPACK)
+#if __has_include(<dlpack/dlpack.h>) && !_CCCL_COMPILER(NVRTC) && !defined(CCCL_DISABLE_DLPACK)
 #  define _CCCL_HAS_DLPACK() 1
 #else // ^^^ has dlpack ^^^ / vvv no dlpack vvv
 #  define _CCCL_HAS_DLPACK() 0
