@@ -165,7 +165,14 @@ public:
     is_nothrow_default_constructible_v<data_handle_type> && is_nothrow_default_constructible_v<mapping_type>
     && is_nothrow_default_constructible_v<accessor_type>)
       : __base()
-  {}
+  {
+    if constexpr (::cuda::std::__has_detect_invalidity<accessor_type>)
+    {
+      [[maybe_unused]] const auto& __tmp = mapping(); // workaround for clang with nodiscard
+      _CCCL_ASSERT(!accessor().__detectably_invalid(data_handle(), __tmp.required_span_size()),
+                   "mdspan: invalid data handle");
+    }
+  }
 
   _CCCL_HIDE_FROM_ABI constexpr mdspan(const mdspan&) = default;
   _CCCL_HIDE_FROM_ABI constexpr mdspan(mdspan&&)      = default;
@@ -174,52 +181,108 @@ public:
   _CCCL_REQUIRES(__constraints::template __can_construct_from_handle_and_variadic<_OtherIndexTypes...>)
   _CCCL_API explicit constexpr mdspan(data_handle_type __p, _OtherIndexTypes... __exts)
       : __base(::cuda::std::move(__p), extents_type(static_cast<index_type>(::cuda::std::move(__exts))...))
-  {}
+  {
+    if constexpr (::cuda::std::__has_detect_invalidity<accessor_type>)
+    {
+      [[maybe_unused]] const auto& __tmp = mapping(); // workaround for clang with nodiscard
+      _CCCL_ASSERT(!accessor().__detectably_invalid(data_handle(), __tmp.required_span_size()),
+                   "mdspan: invalid data handle");
+    }
+  }
 
   _CCCL_TEMPLATE(class _OtherIndexType, size_t _Size)
   _CCCL_REQUIRES(__mdspan_detail::__matches_dynamic_rank<extents_type, _Size> _CCCL_AND
                    __constraints::template __is_constructible_from_index_type<_OtherIndexType>)
   _CCCL_API constexpr mdspan(data_handle_type __p, const array<_OtherIndexType, _Size>& __exts)
       : __base(::cuda::std::move(__p), extents_type{__exts})
-  {}
+  {
+    if constexpr (::cuda::std::__has_detect_invalidity<accessor_type>)
+    {
+      [[maybe_unused]] const auto& __tmp = mapping(); // workaround for clang with nodiscard
+      _CCCL_ASSERT(!accessor().__detectably_invalid(data_handle(), __tmp.required_span_size()),
+                   "mdspan: invalid data handle");
+    }
+  }
 
   _CCCL_TEMPLATE(class _OtherIndexType, size_t _Size)
   _CCCL_REQUIRES(__mdspan_detail::__matches_static_rank<extents_type, _Size> _CCCL_AND
                    __constraints::template __is_constructible_from_index_type<_OtherIndexType>)
   _CCCL_API explicit constexpr mdspan(data_handle_type __p, const array<_OtherIndexType, _Size>& __exts)
       : __base(::cuda::std::move(__p), extents_type{__exts})
-  {}
+  {
+    if constexpr (::cuda::std::__has_detect_invalidity<accessor_type>)
+    {
+      [[maybe_unused]] const auto& __tmp = mapping(); // workaround for clang with nodiscard
+      _CCCL_ASSERT(!accessor().__detectably_invalid(data_handle(), __tmp.required_span_size()),
+                   "mdspan: invalid data handle");
+    }
+  }
 
   _CCCL_TEMPLATE(class _OtherIndexType, size_t _Size)
   _CCCL_REQUIRES(__mdspan_detail::__matches_dynamic_rank<extents_type, _Size> _CCCL_AND
                    __constraints::template __is_constructible_from_index_type<_OtherIndexType>)
   _CCCL_API constexpr mdspan(data_handle_type __p, span<_OtherIndexType, _Size> __exts)
       : __base(::cuda::std::move(__p), extents_type{__exts})
-  {}
+  {
+    if constexpr (::cuda::std::__has_detect_invalidity<accessor_type>)
+    {
+      [[maybe_unused]] const auto& __tmp = mapping(); // workaround for clang with nodiscard
+      _CCCL_ASSERT(!accessor().__detectably_invalid(data_handle(), __tmp.required_span_size()),
+                   "mdspan: invalid data handle");
+    }
+  }
 
   _CCCL_TEMPLATE(class _OtherIndexType, size_t _Size)
   _CCCL_REQUIRES(__mdspan_detail::__matches_static_rank<extents_type, _Size> _CCCL_AND
                    __constraints::template __is_constructible_from_index_type<_OtherIndexType>)
   _CCCL_API explicit constexpr mdspan(data_handle_type __p, span<_OtherIndexType, _Size> __exts)
       : __base(::cuda::std::move(__p), extents_type{__exts})
-  {}
+  {
+    if constexpr (::cuda::std::__has_detect_invalidity<accessor_type>)
+    {
+      [[maybe_unused]] const auto& __tmp = mapping(); // workaround for clang with nodiscard
+      _CCCL_ASSERT(!accessor().__detectably_invalid(data_handle(), __tmp.required_span_size()),
+                   "mdspan: invalid data handle");
+    }
+  }
 
   _CCCL_TEMPLATE(class _AccessorPolicy2 = _AccessorPolicy, class _Mapping2 = mapping_type)
   _CCCL_REQUIRES(
     is_default_constructible_v<_AccessorPolicy2> _CCCL_AND is_constructible_v<_Mapping2, const extents_type&>)
   _CCCL_API constexpr mdspan(data_handle_type __p, const extents_type& __exts)
       : __base(::cuda::std::move(__p), __exts)
-  {}
+  {
+    if constexpr (::cuda::std::__has_detect_invalidity<accessor_type>)
+    {
+      [[maybe_unused]] const auto& __tmp = mapping(); // workaround for clang with nodiscard
+      _CCCL_ASSERT(!accessor().__detectably_invalid(data_handle(), __tmp.required_span_size()),
+                   "mdspan: invalid data handle");
+    }
+  }
 
   _CCCL_TEMPLATE(class _AccessorPolicy2 = _AccessorPolicy)
   _CCCL_REQUIRES(is_default_constructible_v<_AccessorPolicy2>)
   _CCCL_API constexpr mdspan(data_handle_type __p, const mapping_type& __m)
       : __base(::cuda::std::move(__p), __m)
-  {}
+  {
+    if constexpr (::cuda::std::__has_detect_invalidity<accessor_type>)
+    {
+      [[maybe_unused]] const auto& __tmp = mapping(); // workaround for clang with nodiscard
+      _CCCL_ASSERT(!accessor().__detectably_invalid(data_handle(), __tmp.required_span_size()),
+                   "mdspan: invalid data handle");
+    }
+  }
 
   _CCCL_API constexpr mdspan(data_handle_type __p, const mapping_type& __m, const accessor_type& __a)
       : __base(::cuda::std::move(__p), __m, __a)
-  {}
+  {
+    if constexpr (::cuda::std::__has_detect_invalidity<accessor_type>)
+    {
+      [[maybe_unused]] const auto& __tmp = mapping(); // workaround for clang with nodiscard
+      _CCCL_ASSERT(!accessor().__detectably_invalid(data_handle(), __tmp.required_span_size()),
+                   "mdspan: invalid data handle");
+    }
+  }
 
   _CCCL_TEMPLATE(class _OtherElementType, class _OtherExtents, class _OtherLayoutPolicy, class _OtherAccessor)
   _CCCL_REQUIRES(
@@ -233,7 +296,12 @@ public:
                   "mdspan: incompatible data_handle_type for mdspan construction");
     static_assert(is_constructible_v<extents_type, _OtherExtents>,
                   "mdspan: incompatible extents for mdspan construction");
-
+    if constexpr (::cuda::std::__has_detect_invalidity<accessor_type>)
+    {
+      [[maybe_unused]] const auto& __tmp = mapping(); // workaround for clang with nodiscard
+      _CCCL_ASSERT(!accessor().__detectably_invalid(data_handle(), __tmp.required_span_size()),
+                   "mdspan: invalid data handle");
+    }
     if constexpr (extents_type::rank() != 0)
     {
       // The following precondition is part of the standard, but is unlikely to be triggered.
@@ -265,7 +333,12 @@ public:
                   "mdspan: incompatible data_handle_type for mdspan construction");
     static_assert(is_constructible_v<extents_type, _OtherExtents>,
                   "mdspan: incompatible extents for mdspan construction");
-
+    if constexpr (::cuda::std::__has_detect_invalidity<accessor_type>)
+    {
+      [[maybe_unused]] const auto& __tmp = mapping(); // workaround for clang with nodiscard
+      _CCCL_ASSERT(!accessor().__detectably_invalid(data_handle(), __tmp.required_span_size()),
+                   "mdspan: invalid data handle");
+    }
     if constexpr (extents_type::rank() != 0)
     {
       // The following precondition is part of the standard, but is unlikely to be triggered.
@@ -448,24 +521,24 @@ public:
   [[nodiscard]]
   _CCCL_API constexpr bool is_unique() const noexcept(noexcept(::cuda::std::declval<const mapping_type&>().is_unique()))
   {
-    auto __tmp = mapping(); // workaround for clang with nodiscard
+    const auto& __tmp = mapping(); // workaround for clang with nodiscard
     return __tmp.is_unique();
   }
   [[nodiscard]] _CCCL_API constexpr bool is_exhaustive() const
     noexcept(noexcept(::cuda::std::declval<const mapping_type&>().is_exhaustive()))
   {
-    auto __tmp = mapping(); // workaround for clang with nodiscard
+    const auto& __tmp = mapping(); // workaround for clang with nodiscard
     return __tmp.is_exhaustive();
   }
   [[nodiscard]] _CCCL_API constexpr bool is_strided() const
     noexcept(noexcept(::cuda::std::declval<const mapping_type&>().is_strided()))
   {
-    auto __tmp = mapping(); // workaround for clang with nodiscard
+    const auto& __tmp = mapping(); // workaround for clang with nodiscard
     return __tmp.is_strided();
   }
   [[nodiscard]] _CCCL_API constexpr index_type stride(rank_type __r) const
   {
-    auto __tmp = mapping(); // workaround for clang with nodiscard
+    const auto& __tmp = mapping(); // workaround for clang with nodiscard
     return __tmp.stride(__r);
   }
 };
