@@ -1,3 +1,8 @@
+# Copyright (c) 2025-2026, NVIDIA CORPORATION & AFFILIATES. ALL RIGHTS RESERVED.
+#
+#
+# SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
+
 from typing import Callable
 
 import numpy as np
@@ -29,7 +34,6 @@ class _SegmentedReduce:
         "start_offsets_in_cccl",
         "end_offsets_in_cccl",
         "h_init_cccl",
-        "op",
         "op_cccl",
     ]
 
@@ -46,31 +50,11 @@ class _SegmentedReduce:
         self.d_out_cccl = cccl.to_cccl_output_iter(d_out)
         self.start_offsets_in_cccl = cccl.to_cccl_input_iter(start_offsets_in)
         self.end_offsets_in_cccl = cccl.to_cccl_input_iter(end_offsets_in)
-
-        # set host advance functions
-        cccl.cccl_iterator_set_host_advance(self.d_out_cccl, d_out)
-        cccl.cccl_iterator_set_host_advance(
-            self.start_offsets_in_cccl, start_offsets_in
-        )
-        if (
-            self.start_offsets_in_cccl.is_kind_iterator()
-            and self.end_offsets_in_cccl.is_kind_iterator()
-            and isinstance(start_offsets_in, IteratorBase)
-            and isinstance(end_offsets_in, IteratorBase)
-            and start_offsets_in.kind == end_offsets_in.kind
-        ):
-            self.end_offsets_in_cccl.host_advance_fn = (
-                self.start_offsets_in_cccl.host_advance_fn
-            )
-        else:
-            cccl.cccl_iterator_set_host_advance(
-                self.end_offsets_in_cccl, end_offsets_in
-            )
-
         self.h_init_cccl = cccl.to_cccl_value(h_init)
 
         # Compile the op with value types
         value_type = get_value_type(h_init)
+
         self.op_cccl = op.compile((value_type, value_type), value_type)
 
         self.build_result = call_build(
@@ -88,16 +72,26 @@ class _SegmentedReduce:
         temp_storage,
         d_in,
         d_out,
+        op: Callable | OpAdapter,
         num_segments: int,
         start_offsets_in,
         end_offsets_in,
         h_init,
         stream=None,
     ):
+        if num_segments > np.iinfo(np.int32).max:
+            raise RuntimeError(
+                "Segmented sort does not currently support more than 2^31-1 segments."
+            )
         set_cccl_iterator_state(self.d_in_cccl, d_in)
         set_cccl_iterator_state(self.d_out_cccl, d_out)
         set_cccl_iterator_state(self.start_offsets_in_cccl, start_offsets_in)
         set_cccl_iterator_state(self.end_offsets_in_cccl, end_offsets_in)
+
+        # Update op state for stateful ops
+        op_adapter = make_op_adapter(op)
+        op_adapter.update_op_state(self.op_cccl)
+
         self.h_init_cccl.state = to_cccl_value_state(h_init)
 
         stream_handle = validate_and_get_stream(stream)
@@ -200,6 +194,7 @@ def segmented_reduce(
         None,
         d_in,
         d_out,
+        op,
         num_segments,
         start_offsets_in,
         end_offsets_in,
@@ -211,6 +206,7 @@ def segmented_reduce(
         tmp_storage,
         d_in,
         d_out,
+        op,
         num_segments,
         start_offsets_in,
         end_offsets_in,
