@@ -15,7 +15,7 @@ from ._three_way_partition import make_three_way_partition
 
 
 class _Select:
-    __slots__ = ["partitioner", "discard_second", "discard_unselected"]
+    __slots__ = ["partitioner", "discard_second", "discard_unselected", "false_op"]
 
     def __init__(
         self,
@@ -29,6 +29,9 @@ class _Select:
         self.discard_second = DiscardIterator(d_out)
         self.discard_unselected = DiscardIterator(d_out)
 
+        # Create adapter for the always-false second predicate
+        self.false_op = make_op_adapter(lambda x: False)
+
         # Use three_way_partition internally
         self.partitioner = make_three_way_partition(
             d_in,
@@ -37,7 +40,7 @@ class _Select:
             self.discard_unselected,  # unselected_out - discarded
             d_num_selected_out,
             cond,  # select_first_part_op - user's select condition
-            lambda x: False,  # select_second_part_op - always false
+            self.false_op,  # select_second_part_op - always false
         )
 
     def __call__(
@@ -46,6 +49,7 @@ class _Select:
         d_in,
         d_out,
         d_num_selected_out,
+        cond,
         num_items: int,
         stream=None,
     ):
@@ -56,6 +60,8 @@ class _Select:
             self.discard_second,
             self.discard_unselected,
             d_num_selected_out,
+            make_op_adapter(cond),
+            self.false_op,
             num_items,
             stream,
         )
@@ -143,13 +149,16 @@ def select(
         num_items: Number of items in the input sequence.
         stream: CUDA stream to use for the operation (optional).
     """
-    selector = make_select(d_in, d_out, d_num_selected_out, cond)
+    # Create adapter to support stateful ops
+    cond_adapter = make_op_adapter(cond)
+    selector = make_select(d_in, d_out, d_num_selected_out, cond_adapter)
 
     tmp_storage_bytes = selector(
         None,
         d_in,
         d_out,
         d_num_selected_out,
+        cond_adapter,
         num_items,
         stream,
     )
@@ -159,6 +168,7 @@ def select(
         d_in,
         d_out,
         d_num_selected_out,
+        cond_adapter,
         num_items,
         stream,
     )
