@@ -1,16 +1,14 @@
-# Copyright (c) 2025, NVIDIA CORPORATION & AFFILIATES. ALL RIGHTS RESERVED.
+# Copyright (c) 2025-2026, NVIDIA CORPORATION & AFFILIATES. ALL RIGHTS RESERVED.
+#
 #
 # SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
 
 import ctypes
 
-from llvmlite import ir  # noqa: F401
-from numba import cuda, types  # noqa: F401
-from numba.core.datamodel.registry import default_manager  # noqa: F401
-from numba.core.extending import as_numba_type, intrinsic  # noqa: F401
+from numba import cuda
 
+from .. import types
 from .._caching import cache_with_registered_key_functions
-from ..struct import make_struct_type
 from ._iterators import (
     IteratorBase,
     IteratorKind,
@@ -19,35 +17,49 @@ from ._iterators import (
 
 
 class ZipIteratorKind(IteratorKind):
-    pass
+    def __init__(
+        self,
+        value_type: types.TypeDescriptor,
+        iterators_kinds: tuple[IteratorKind, ...],
+    ):
+        self.iterators_kinds = iterators_kinds
+        self.value_type = value_type
+
+    def __repr__(self):
+        return f"ZipIteratorKind({self.value_type}, {self.iterators_kinds})"
+
+    def __eq__(self, other):
+        return (
+            type(self) is type(other)
+            and self.value_type == other.value_type
+            and self.iterators_kinds == other.iterators_kinds
+        )
+
+    def __hash__(self):
+        return hash((type(self), self.value_type, self.iterators_kinds))
 
 
 def _get_zip_iterator_metadata(iterators):
-    # return the cvalue, state_type, and value_type for the zip iterator
-    # composed of the input iterators
+    from ..struct import gpu_struct
 
-    n_iterators = len(iterators)
-
-    # ctypes struct for the combined state:
+    # Create ctypes struct for cvalue
     fields = [(f"iter_{i}", it.cvalue.__class__) for i, it in enumerate(iterators)]
     ZipCValueStruct = type("ZipCValueStruct", (ctypes.Structure,), {"_fields_": fields})
 
-    # this iterator's state is a struct composed of the states of the input iterators:
-    state_field_names = tuple(f"state_{i}" for i in range(n_iterators))
-    state_field_types = tuple(it.state_type for it in iterators)
-    ZipState = make_struct_type("ZipState", state_field_names, state_field_types)
+    # Create state struct using gpu_struct
+    state_fields = {f"state_{i}": it.state_type for i, it in enumerate(iterators)}
+    ZipState = gpu_struct(state_fields, name="ZipState")
+    state_type = ZipState._type_descriptor
 
-    # this iterator's value is a struct composed of the values of the input iterators:
-    value_field_names = tuple(f"value_{i}" for i in range(n_iterators))
-    value_field_types = tuple(it.value_type for it in iterators)
-    ZipValue = make_struct_type("ZipValue", value_field_names, value_field_types)
+    # Create value struct using gpu_struct
+    value_fields = {f"value_{i}": it.value_type for i, it in enumerate(iterators)}
+    ZipValue = gpu_struct(value_fields, name="ZipValue")
+    value_type = ZipValue._type_descriptor
 
     cvalue = ZipCValueStruct(
         **{f"iter_{i}": it.cvalue for i, it in enumerate(iterators)}
     )
-    state_type = as_numba_type(ZipState)
-    value_type = as_numba_type(ZipValue)
-    return cvalue, state_type, value_type, ZipValue
+    return cvalue, state_type, value_type
 
 
 # Automatic: iterators tuple → (kind, kind, ...) or (dtype, dtype, ...)
@@ -59,44 +71,22 @@ def _get_advance_and_dereference_functions(iterators):
     # iterator, and the dereference method invokes the dereference method of each
     # input iterator.
 
+    from .._intrinsics import make_struct_field_ptr_intrinsic
+
     n_iterators = len(iterators)
 
     # Create a local namespace for this zip iterator to avoid polluting globals
     # and prevent name collisions when nesting zip iterators
-    local_ns = {
-        "intrinsic": intrinsic,
-        "types": types,
-        "ir": ir,
-        "default_manager": default_manager,
-    }
+    local_ns = {}
 
     # Within the advance and dereference methods of this iterator, we need a way
     # to get pointers to the fields of the state struct (advance and dereference),
-    # and the value struct (dereference). This needs `n` custom intrinsics, one
-    # for each field. The loop below defines those intrinsics:
+    # and the value struct (dereference). This needs `n` intrinsics, one
+    # for each field. Create them using the factory:
     for field_idx in range(n_iterators):
-        func_name = f"get_field_ptr_{field_idx}"
-
-        # Create the intrinsic function
-        intrinsic_code = f"""
-@intrinsic
-def {func_name}(context, struct_ptr_type):
-    def codegen(context, builder, sig, args):
-        struct_ptr = args[0]
-        # Use GEP to get pointer to field at index {field_idx}
-        field_ptr = builder.gep(
-            struct_ptr,
-            [ir.Constant(ir.IntType(32), 0), ir.Constant(
-                ir.IntType(32), {field_idx})],
+        local_ns[f"get_field_ptr_{field_idx}"] = make_struct_field_ptr_intrinsic(
+            field_idx
         )
-        return field_ptr
-
-    struct_model = default_manager.lookup(struct_ptr_type.dtype)
-    field_type = struct_model._members[{field_idx}]
-    return types.CPointer(field_type)(struct_ptr_type), codegen
-"""
-        # Execute the code to create the intrinsic function in local namespace
-        exec(intrinsic_code, local_ns)
 
     # Now we can define the advance and dereference methods of this iterator,
     # which also need to be defined dynamically because they will use the
@@ -188,8 +178,6 @@ def make_zip_iterator(*iterators):
     Returns:
         ZipIterator: Iterator that combines all input iterators
     """
-    from .._cccl_interop import get_value_type
-
     if len(iterators) < 1:
         raise ValueError("At least 1 iterator is required")
 
@@ -197,7 +185,7 @@ def make_zip_iterator(*iterators):
     processed_iterators = []
     for it in iterators:
         if hasattr(it, "__cuda_array_interface__"):
-            it = pointer(it, get_value_type(it))
+            it = pointer(it)
         processed_iterators.append(it)
 
     # Validate all are iterators
@@ -205,9 +193,7 @@ def make_zip_iterator(*iterators):
         if not isinstance(it, IteratorBase):
             raise TypeError(f"Argument {i} must be an iterator or device array")
 
-    cvalue, state_type, value_type, ZipValue = _get_zip_iterator_metadata(
-        processed_iterators
-    )
+    cvalue, state_type, value_type = _get_zip_iterator_metadata(processed_iterators)
 
     advance_func, input_dereference_func, output_dereference_func = (
         _get_advance_and_dereference_functions(processed_iterators)
@@ -223,21 +209,18 @@ def make_zip_iterator(*iterators):
     all_support_output = all(supports_output(it) for it in processed_iterators)
 
     class ZipIterator(IteratorBase):
-        iterator_kind_type = ZipIteratorKind
-
         def __init__(self, iterators_list):
             self._iterators = iterators_list
             self._n_iterators = len(iterators_list)
 
             kinds = [it.kind for it in iterators_list]
+            kind = ZipIteratorKind(value_type, tuple(kinds))
             super().__init__(
+                kind=kind,
                 cvalue=cvalue,
                 state_type=state_type,
                 value_type=value_type,
             )
-            self._kind = self.__class__.iterator_kind_type(
-                (value_type, *kinds), self.state_type
-            )
 
         @property
         def host_advance(self):
@@ -260,4 +243,19 @@ def make_zip_iterator(*iterators):
                 )
             return output_dereference_func
 
+        @property
+        def children(self):
+            return tuple(self._iterators)
+
+        def _rebuild_value_type_from_children(self):
+            from ..struct import gpu_struct
+
+            if not self._iterators:
+                raise ValueError("Zip iterator has no children")
+            value_fields = {
+                f"value_{i}": child.value_type
+                for i, child in enumerate(self._iterators)
+            }
+            self.value_type = gpu_struct(value_fields, name="ZipValue")._type_descriptor
+
     return ZipIterator(processed_iterators)
